// Source code to the Ball Shooting Program.
//
// Author: Kelvin Sung
// Copyright: University of Washington, Bothell
//


//
// Circle Functionality, this is completely independent from GraphicsAPI
//
#include "CirclePrimitive.h"


CirclePrimitive::CirclePrimitive(float x, float y) :
  Center_X(x), Center_Y(y),
	Velocity_X(0.0f), Velocity_Y(0.0f),
	Radius(0.0f)
{
}

void CirclePrimitive::SetRadiusTo(float x, float y)
{
  float dx = Center_X - x;
  float dy = Center_Y - y;
  Radius = sqrtf(dx*dx + dy*dy);
	Velocity_X = (Center_X - x) / UpdatesPerSecond;
	Velocity_Y = (Center_Y - y) / UpdatesPerSecond;
}

void CirclePrimitive::UpdatePosition()
{
	Center_X += Velocity_X;
	Center_Y += Velocity_Y;
}


bool CirclePrimitive::CollideWith(CirclePrimitive *c)
{
	float dx = Center_X - c->Center_X;
	float dy = Center_Y - c->Center_Y;
	float dist = sqrtf(dx*dx + dy*dy);
	bool collide = ( dist < (Radius + c->Radius) );
	if (collide) {
		// perfectly elastic collision with same mess ...
		// incoming vector is (ix, iy), cross-section vector is (cy, cx)
		// normalize
		float ix = dx / dist;
		float iy = dy / dist;
		float cx = -iy;
		float cy = ix;

		// to avoid balls "sticking" to each other, we will push them apart a little
		Center_X = c->Center_X + ix * (Radius + c->Radius);
		Center_Y = c->Center_Y + iy * (Radius + c->Radius);

		//
		// for this->, magnitude in incoming direction
		float md = Velocity_X * ix + Velocity_Y * iy;		// magnitude in i (v dot i)
		float mc = Velocity_X * cx + Velocity_Y * cy;		// magnitude in c (v dot c)
		Velocity_X = md * ix + mc * cx; // (v = md * i + mc * c)
		Velocity_Y = md * iy + mc * cy;

		// for the colliding circle
		md = c->Velocity_X * ix + c->Velocity_Y * iy;
		mc = c->Velocity_X * cx + c->Velocity_Y * cy;
		c->Velocity_X = -md * ix + mc * cx;
		c->Velocity_Y = -md * iy + mc * cy;
	}
	return collide;
}

bool CirclePrimitive::PointIsInCircle(float x, float y)
{
	float dx = Center_X - x;
	float dy = Center_Y - y;
	float dist = sqrtf(dx*dx + dy*dy);
	return ( dist < Radius );
}

bool CirclePrimitive::OutOfBound(float mx, float my)
{
	float minx = Center_X - Radius;
	float miny = Center_Y - Radius;
	float maxx = Center_X + Radius;
	float maxy = Center_Y + Radius;

	return (  (maxx < 0)  || (minx > mx) || (maxy < 0) || (miny > my) );
}
