// Source code to the Ball Shooting Program.
//
// Author: Kelvin Sung
// Copyright: University of Washington, Bothell
//

#ifdef MFC_D3D
#include "../../MFC_D3D/StdAfx.h"
#endif
#include "Model.h"


#define Gravity 	(9.8f/(UpdatesPerSecond * GravityFudgeFactor))  



void Model::UpdateSimulation()
{

	// 1. All Balls free fall under gravitational force and change positions
	std::vector<CirclePrimitive*>::iterator it;
	it = AllWorldBalls.begin();
	while ( (it != AllWorldBalls.end()) ) {
		(*it)->VelocityY() -= Gravity;
		(*it)->UpdatePosition();
		++it;
	}

	// 2. Collide all balls
	it = AllWorldBalls.begin();
	while ( (it != AllWorldBalls.end()) ) {
		// for each circle
		CirclePrimitive *c = (*it);

		// intersect with every other balls in the world
		std::vector<CirclePrimitive*>::iterator iit;
		iit = AllWorldBalls.begin();
		while ( iit != AllWorldBalls.end() ) {
			CirclePrimitive *tc = (*iit);
			// don't collide with self
			if (c != tc) 
				c->CollideWith(tc);
			++iit;
		}
		++it;
	}

	// 3. Delete all those that are out of bound
	// for simplicity, delete from the back ...
  for (int i = (int) (AllWorldBalls.size()-1); i>=0; i--) {
    CirclePrimitive *c = AllWorldBalls[i];
    if (c->OutOfBound(WorldWidth, WorldHeight) ) {
			if (c == HeroBall) 
				HeroBall = NULL;

      delete c;
      it = AllWorldBalls.begin() + i;
      AllWorldBalls.erase(it);
    }
  }
}
