// Source code to the Ball Shooting Program.
//
// Author: Kelvin Sung
// Copyright: University of Washington, Bothell
//


//
// Drawing of Circle, this is completely DEPENDENT on GraphicsAPI
//
#include "../../Common/GraphicsAPI/CirclePrimitive.h"
#include "DeviceVertexFormat.h"

void CirclePrimitive::DrawCircle(GraphicsDevice deviceRegion)
{
  if (Radius <= 0.0f) return;
  if(deviceRegion==NULL) return;

  float pointSize=2.0f;
  deviceRegion->SetRenderState( D3DRS_POINTSIZE, *((DWORD*)&pointSize));
  deviceRegion->SetFVF(DeviceVertexFormatCode);

  // We are going to draw this circle with TriangleFan
  const int kNumPts = 100;
  DeviceVertexFormat v[kNumPts+2];
  float delta = (float)((2.0f * M_PI) / kNumPts);
  v[0].m_Point.x = Center_X;
  v[0].m_Point.y = Center_Y;
  v[0].m_Point.z = 0.0f;
  v[0].m_Color = 0xFFFFFFFF; // white

  D3DCOLOR c = D3DCOLOR_COLORVALUE(0.7f, 0.6f, 0.6f, 0.0f);
  for (int i=0; i<(kNumPts); i++)
  {
    v[i+1].m_Point.x = Center_X + (Radius * sinf(i*delta));
    v[i+1].m_Point.y = Center_Y + (Radius * cosf(i*delta));
    v[i+1].m_Point.z = 0.0f;
    v[i+1].m_Color = c;
  }
  v[kNumPts+1] = v[1];

  deviceRegion->DrawPrimitiveUP(D3DPT_TRIANGLEFAN, kNumPts, (CONST void *)v, sizeof(DeviceVertexFormat));
}
