// Source code to the Ball Shooting Program.
//
// Author: Kelvin Sung
// Copyright: University of Washington, Bothell
//


// MainUIWindow.cpp : implementation file
//

#include "stdafx.h"
#include "MfcD3dSolution.h"
#include "MainUIWindow.h"
#include "ViewController/CWndD3D.h"

// To use GraphicsAPI functionality
#include "GraphicsAPI/GraphicsAPI.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	enum { IDD = IDD_ABOUTBOX };

	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

// Implementation
protected:
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
END_MESSAGE_MAP()


// MainUIWindow dialog



MainUIWindow::MainUIWindow(CWnd* pParent /*=NULL*/)
	: CDialog(MainUIWindow::IDD, pParent)
	, EchoArea(_T(""))
	, VelocityXEcho(_T(""))
	, VelocityYEcho(_T(""))
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void MainUIWindow::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Text(pDX, IDC_STATUS_AREA, EchoArea);
	DDX_Text(pDX, IDC_X_SLIDER_ECHO, VelocityXEcho);
	DDX_Text(pDX, IDC_Y_SLIDER_ECHO, VelocityYEcho);
	DDX_Control(pDX, IDC_X_SLIDER_BAR, VelocityXSlider);
	DDX_Control(pDX, IDC_Y_SLIDER_BAR, VelocityYSlider);
}

BEGIN_MESSAGE_MAP(MainUIWindow, CDialog)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDCANCEL, OnBnClickedQuit)
	ON_WM_HSCROLL()
	ON_WM_TIMER()
END_MESSAGE_MAP()


// MainUIWindow message handlers

BOOL MainUIWindow::OnInitDialog()
{
	CDialog::OnInitDialog();
	
	// Add "About..." menu item to system menu.

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	
    // Now create the abstract D3d output pipeline connection
	if( NULL == ( D3D = Direct3DCreate9( D3D_SDK_VERSION ) ) ) {
		MessageBox("D3d Creation Failed (1)!");
		return FALSE;
	}

	LargeView.PlaceViewOnReserouce(this, IDC_D3DX_PLACEHOLDER);
	CRect rect;
	LargeView.GetWindowRect(&rect);
	float worldWidth = float(rect.Width());
	float worldHeight = float (rect.Height());

	SmallView.PlaceViewOnReserouce(this, IDC_SMALL_D3DX_PLACEHOLDER);

	// d. Initialize the Direct3D Device
	//
		
		if (!LargeView.InitD3D(D3D, worldWidth, worldHeight)){
			MessageBox("D3D device initialization failed!");
		}

		if (!SmallView.InitD3D(D3D, worldWidth, worldHeight)) {
			MessageBox("D3D device initialization failed on Small LargeView!");
		}

		LargeView.SetModel(&TheModel);
		SmallView.SetModel(&TheModel);
		TheModel.SetWorldBound(worldWidth, worldHeight);


	VelocityXSlider.InitializeSliderWithEcho(&VelocityXEcho, -100, 100, 0);
	VelocityYSlider.InitializeSliderWithEcho(&VelocityYEcho, -100, 100, 0);

	//		Here we initialize the timer, to go off every 25 milli seconds
	//		when timer "goes-off", our OnTimer() function will be called and
	//		it is upto us to decide what we want to do.
	//
	const static int msInterval = 25;	// timer going off every 25 milli seconds.
  	SetTimer(0, msInterval, NULL);		// our goal is to approx 40 frames per second


	UpdateData(false);
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

void MainUIWindow::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void MainUIWindow::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR MainUIWindow::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

void MainUIWindow::OnBnClickedQuit()
{
	OnCancel();
}

void MainUIWindow::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
	// Do super class work first
	CDialog::OnHScroll(nSBCode, nPos, pScrollBar);

	VelocityXSlider.UpdateSliderEcho();
	VelocityYSlider.UpdateSliderEcho();

	float vx = VelocityXSlider.GetSliderValue();
	float vy = VelocityYSlider.GetSliderValue();

	TheModel.SetHeroBallVelocity(vx, vy);
}

void MainUIWindow::OnTimer(UINT nIDEvent)
{

	TheModel.UpdateSimulation();

	if (TheModel.HeroBallExists()) {
			float vx = TheModel.HeroVelocityX();
			float vy = TheModel.HeroVelocityY();
			VelocityXSlider.SetSliderValue(vx);
			VelocityYSlider.SetSliderValue(vy);
			VelocityXSlider.UpdateSliderEcho();
			VelocityYSlider.UpdateSliderEcho();
	}
	
	LargeView.RedrawView();
	SmallView.RedrawView();

	int n = TheModel.NumBallsOnScreen();
	EchoArea.Format("Number of Balls=%d", n);
	UpdateData(FALSE);
}
