// Source code to the Ball Shooting Program.
//
// Author: Kelvin Sung
// Copyright: University of Washington, Bothell
//


#pragma once

// SliderCtrlWithEcho
// 		1. Support echoing of slider value to the a static text
// 		2. Support floating point slider range (with resolution of 100K)

class SliderCtrlWithEcho : public CSliderCtrl
{
	DECLARE_DYNAMIC(SliderCtrlWithEcho)

public:
	SliderCtrlWithEcho();
	virtual ~SliderCtrlWithEcho();

	void	InitializeSliderWithEcho(CString *echo, float min, float max, float init);
	void	SetSliderValue(float userValue) { SetPos(userToMFCPos(userValue) ); UpdateData(true); }
	float	GetSliderValue() { return mfcToUserPos(GetPos()); }
	void  	UpdateSliderEcho() { SliderEcho->Format("%.2f", mfcToUserPos(GetPos())); }

protected:
	DECLARE_MESSAGE_MAP()
	
	int		userToMFCPos(float userValue) { 
					return int(((userValue - MinRange) * kMaxSliderRange / TotalRange)+0.5f); 
			}
	float	mfcToUserPos(int mfcValue) { return (MinRange + ( (float(mfcValue) / float(kMaxSliderRange)) * TotalRange)); }
	void  	UpdateSliderEcho(float userValue) { SliderEcho->Format("%.2f", userValue); }
	

private:
	static const int kMaxSliderRange = 100000;

	CString		*SliderEcho;	// This is where the slider's echo will show up

	float		MinRange;		// Min Range number
	float		TotalRange;	// Total range covered from min to max
};


